Package: tximport
Version: 1.40.0
Title: Import and summarize transcript-level estimates for transcript-
        and gene-level analysis
Description: Imports transcript-level abundance, estimated counts and
        transcript lengths, and summarizes into matrices for use with
        downstream gene-level analysis packages. Average transcript
        length, weighted by sample-specific transcript abundance
        estimates, is provided as a matrix which can be used as an
        offset for different expression of gene-level counts.
Author: Michael Love [cre,aut], Charlotte Soneson [aut], Mark Robinson
        [aut], Rob Patro [ctb], Andrew Parker Morgan [ctb], Ryan C.
        Thompson [ctb], Matt Shirley [ctb], Avi Srivastava [ctb]
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
License: LGPL (>=2)
VignetteBuilder: knitr
Imports: utils, stats, methods
Suggests: knitr, rmarkdown, testthat, tximportData,
        TxDb.Hsapiens.UCSC.hg19.knownGene, readr (>= 0.2.2), arrow,
        limma, edgeR (>= 4.9.2), DESeq2 (>= 1.11.6), rhdf5, jsonlite,
        matrixStats, Matrix, eds
URL: https://github.com/thelovelab/tximport
biocViews: DataImport, Preprocessing, RNASeq, Transcriptomics,
        Transcription, GeneExpression, ImmunoOncology
RoxygenNote: 7.2.3
NeedsCompilation: no
Encoding: UTF-8
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:42:36 UTC
RemoteUrl: https://github.com/bioc/tximport
RemoteRef: RELEASE_3_23
RemoteSha: 793a7843e7dd65a0d60fdcb8727027bb133c6127
Packaged: 2026-04-29 14:58:25 UTC; root
Built: R 4.6.0; ; 2026-04-29 15:02:30 UTC; unix
